# include	<stdio.h>
# include	<stdlib.h>

# include	"b3.h"
# include	"vcrplus.h"
# include	"ileave.h"
# include	"scramble.h"
# include	"misc.h"
# include	"top456.h"

long	debug = 0;


static char *mon[] = {  "Jan", "Feb", "Mar", "Apr", "May", "Jun",
			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

static int	trial( int month, int day, int year, int channel );


static void
usage( char *progname )
{

# ifdef US
	printf("Usage: %s <Month (1-12)> <Day (1-12)> <Year (0-99)> <Channel (1-48)>\n",
				progname);
# endif
# ifdef UK
	printf("Usage: %s <Day (1-31)> <Month (1-12)> <Year (0-99)> <Channel (1-16)>\n",
				progname);
# endif

	return;
}


int
main( int argc, char *argv[] )
{
	int	month, day, year, channel;

	if (argc != 5) {
		usage("confirm");
		return -1;
	}

# ifdef US
	month = atoi(argv[1]);
	day = atoi(argv[2]);
# endif
# ifdef UK
	day = atoi(argv[1]);
	month = atoi(argv[2]);
# endif
	year = atoi(argv[3]);
	channel = atoi(argv[4]);

	if ((month > 12) || (month < 1) ||
	     (day < 1) || (day > 31)) {
		printf("Invalid date\n");
		usage("confirm");
		return -1;
	}

# ifdef US
	if ((channel < 1) || (channel > 48)) {
# endif
# ifdef UK
	if ((channel < 1) || (channel > 16)) {
# endif
		printf("Invalid channel number\n");
		usage("confirm");
		return -1;
	}

	while (trial(month, day, year, channel));
	dump();
	return 0;
}



static int
trial( int month, int day, int year, int channel )
{
	int	j;
	long	tblidx, limit;
# ifdef US
	long	doneflag, trailers;
# endif
	long	s1_out, top5, newspaper, s4_out;
	int	starttime, duration, cval, s5_out, ofout, bot3, rem;

	/* get the  t  bits and the  c  bits */
	cval = channel - 1;

	tblidx = getguesses();
	printf("Index = %ld\n", tblidx);
	lookup(tblidx, &starttime, &duration);

	year = year % 100 ;

	/* from them infer what must have been step 4 & step 5 results */
	interleave(tblidx, cval, (&s4_out), (&s5_out)) ;

	/* if the mapped_top is zero then top and offset are zero */
	if (s4_out == 0) {
		top5 = 0;
		ofout = 0;
	}
	else {
# ifdef FULLSEARCH
		/* find the smallest unmapped_top giving correct mapped_top */
		nd(s4_out, &limit);
		top5 = limit / 10;
		printf("S4=%ld, top5 = %ld, limit = %d\n", s4_out, top5, limit);
		doneflag = 0;

		while (doneflag == 0) {
			offset(day, year, top5, (&ofout), (&j)) ;
			putchar((top5 & 0x01)? '+': 'X');
			putchar('\r');

			if (j == s4_out) {
				printf("\nFound it: %ld\n", top5);
				break;
			}

			if (++top5 >= limit) {
				printf("Can't seem to find an encoding - sorry\n");
				break;
			}
		}
# else
		int	i;
		long	l;

		if ((j = nd(s4_out, &limit)) > 3) {
			printf("That needs a 7 or 8 digit code - sorry\n");
			return -1;
		}

		limit /= 10;
		ofout = 0;
		top5 = s4_out;

		/* get a top5 with same no digits as s4_out */
		/* may have to loop several times */
		do {
			/* reverse the map_top encryption */
			top5 = unmap_top(day, year, top5, j);

			/* use that 'top5' to find offset */
			for(i=0; i<=(year%16); i++)
				ofout += ( int )(map_top(day, i, top5, j) % 10);
# ifdef DEBUG
			printf("%ld\n", top5);
# endif
		} while (top5 < limit);

		/* add sum of final top5's digits to offset */
		l = top5;

		while (l > 0) {
			ofout += ( int )(l % 10);
			l /= 10;
		}

		ofout %= 32;
# endif
	}
		
	/* have two of the three inputs to step 5; determine the rem */
	for(rem=0; rem<32; rem++)
	{
		j = (rem + (day*(month+1)) + ofout) % 32 ;
		if(j == s5_out) break ;
	}

	/* assemble the output of step 1 */
	bot3 = encode_bot3(day, rem);
	s1_out = bot3 + (1000 * top5) ;
# ifdef DEBUG
	printf("s1=%ld\n", s1_out);
# endif

	/* invert the mixing */
	newspaper = f1(s1_out, KEY002);

	printf("\t%s  %2d  %2d\n", mon[month-1], day, year);
	printf("\tChannel= %2d\n", channel);
	printf("\tStart time= %4d  Length= %3d\n", starttime, duration);
	printf("\tEnd time= %3d\n\n", end_time(starttime, duration));
# ifdef UK
	printf("\tVideoPlus code= %8ld\n", newspaper);
# endif
# ifdef US
	printf("\tVCR+ code= %8ld\n", newspaper);
# endif
	return confirm(tblidx) != 'q';
}
