# include	<stdio.h>
# include	<stdlib.h>
# include	<malloc.h>
# include	<string.h>

# include	"vcrplus.h"
# include	"fnc7.h"
# include	"ileave.h"

# define	OLD		/* one line of durations per start-time */

struct dur {
	int		tdur;
	struct dur	*dnext;
};

struct start {
	int		tstart;
	struct dur	*dlist;
	struct start	*next;
};

struct start *head = NULL;

int	s4out_year_month( fnc *aptr, fnc *bptr );
void	request( long s4out );
void	addrequest( int start, int duration );
void	listrequests( void );
void	freerequests( void );
void	usage( void );

main( int argc, char *argv[] )
{
	int		found, year, month;
	long		s4out;
	fnc		*fptr;

	if (argc != 3) {
		usage();
		exit(-1);
	}

	readtables();
	killduptop5();
	fptr = reorder7(s4out_year_month);

	year = atoi(argv[2]);
	month = atoi(argv[1]);

	while (fptr) {
		found = 0;
		s4out = fptr->s4_out & S4_BITS;
		while (fptr && (fptr->s4_out & S4_BITS) == s4out) {
			if (fptr->year == year && fptr->month == month)
				found = 1;

			fptr = fptr->next;
		}
		
		if (!found)
			request(s4out);
	}

# ifdef OLD
	listrequests();
	freerequests();
# endif
	return 0;
}

void
usage( void )
{

	printf("Usage: missing <month> <year>\n");
	return;
}


void
listrequests( void )
{
	struct start	*sptr;
	struct dur	*dptr;

	for (sptr = head; sptr; sptr = sptr->next) {
		printf("Start: %04d, Duration: ", sptr->tstart);

		for (dptr = sptr->dlist; dptr; dptr = dptr->dnext)
			printf("%d,", dptr->tdur);

		putchar('\n');
	}
			
	putchar('\n');
	return;
}


void
freerequests( void )
{
	struct start	*sptr = head, *psptr;
	struct dur	*dptr, *pdptr;

	while (psptr = sptr) {
		dptr = sptr->dlist;
		while (pdptr = dptr) {
			dptr = dptr->dnext;
			free(( void * )pdptr);
		}

		sptr = sptr->next;
		free(( void * ) psptr);
	}
		
	head = NULL;
	return;
}


int
s4out_year_month( fnc *aptr, fnc *bptr )
{
	long	ret;

	if ((ret = aptr->s4_out - bptr->s4_out) == 0)
		if ((ret = aptr->year - bptr->year) == 0)
			ret = aptr->month - bptr->month;

	return ( int ) ret;
}


void
request( long s4out )
{
	int	cval, start, dur, i;
	long	tval;

	de_interleave( s4out, 0, &tval, &cval);

# ifndef OLD
	printf("Index = %04ld:\n", tval);
# endif

	for (i = 0; i < 8; i++) {
		lookup(tval+i, &start, &dur);
		addrequest(start, dur);
	}

# ifndef OLD
	listrequests();
	freerequests();
# endif
	return;
}


void
addrequest( int start, int dur )
{
	struct start	*sptr = head, *psptr = NULL;
	struct dur	*dptr, *pdptr = NULL;

	if (sptr == NULL || sptr->tstart > start) {
		/* special: create new first-entry */
		head = ( struct start * ) malloc(sizeof( struct start ));
		head->tstart = start;
		head->dlist = ( struct dur * ) malloc(sizeof( struct dur ));
		head->dlist->tdur = dur;
		head->dlist->dnext = NULL;
		head->next = sptr;
		return;
	}

	/* search for start time */
	while (sptr) {
		if (sptr->tstart == start) {
			/* found start time */
			/* add duration to list */
			dptr = sptr->dlist;

			if (dptr->tdur > dur) {
				/* special: create new first-entry */
				sptr->dlist = ( struct dur * ) malloc(sizeof( struct dur ));
				sptr->dlist->tdur = dur;
				sptr->dlist->dnext = dptr;
				return;
			}

			/* search for duration */
			while (dptr) {
				if (dptr->tdur == dur) {
					printf("Duplicate %d/%d?\n", start, dur);
					return;
				}
				else if (dptr->tdur > dur) {
					/* insert here */
					pdptr->dnext = ( struct dur * ) malloc(sizeof( struct dur ));
					pdptr->dnext->tdur = dur;
					pdptr->dnext->dnext = dptr;
					return;
				}

				pdptr = dptr;
				dptr = dptr->dnext;
			}

			/* insert at end */
			pdptr->dnext = ( struct dur * ) malloc(sizeof( struct dur ));
			pdptr->dnext->tdur = dur;
			pdptr->dnext->dnext = dptr;
			return;
		}
		else if (sptr->tstart > start) {
			/* insert here */
			psptr->next = ( struct start * ) malloc(sizeof( struct start ));
			psptr->next->tstart = start;
			psptr->next->dlist = ( struct dur * ) malloc(sizeof( struct dur ));
			psptr->next->dlist->tdur = dur;
			psptr->next->dlist->dnext = NULL;
			psptr->next->next = sptr;
			return;
		}

		psptr = sptr;
		sptr = sptr->next;
	}

	/* insert at end */
	psptr->next = ( struct start * ) malloc(sizeof( struct start ));
	psptr->next->tstart = start;
	psptr->next->dlist = ( struct dur * ) malloc(sizeof( struct dur ));
	psptr->next->dlist->tdur = dur;
	psptr->next->dlist->dnext = NULL;
	psptr->next->next = NULL;
	return;
}
